
// $Id: PixmapList.cpp,v 1.20 2013/02/13 10:28:57 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*******************************************************************************/

#include "PixmapEngine.h"
#include "PixmapList.h"
#include "SeparatorPixmap.h"
#include "XmlOptions.h"

#include <QtGui/QPainter>

//_________________________________________________________________________
PixmapList::PixmapList(void):
    Counter( "PixmapList" ),
    maxSize_( 0 ),
    minSize_( 0 ),
    sizeIncrement_( 0 ),
    maxSizeIndex_( 0 ),
    maxHighlightIndex_( 0 ),
    fullIntensity_( 0 )
{}

//_________________________________________________________________________
void PixmapList::setAttributes( const TaskBarAttributes& taskBarAttributes, const TaskAttributes& attributes )
{

    Debug::Throw( "PixmapList::setAttributes.\n" );

    taskBarAttributes_ = taskBarAttributes;
    attributes_ = attributes;

    // size
    minSize_ = XmlOptions::get().get<int>( "MIN_BUTTON_SIZE" );
    maxSize_ = XmlOptions::get().get<int>( "MAX_BUTTON_SIZE" );
    maxSize_ = qMax( maxSize_, minSize_ );
    maxSizeIndex_ = maxSize_ - minSize_ + 1;

    // if zoom is disabled, only store min and max size
    if( !taskBarAttributes.hasFlag( TaskBarAttributes::ZoomEnabled ) && taskBarAttributes.mode() != TaskBarAttributes::Parabolic )
    { maxSizeIndex_ = qMin( maxSizeIndex_, (int)2 ); }

    sizeIncrement_ = maxSizeIndex_ > 1 ? (maxSize_-minSize_)/(maxSizeIndex_-1):0;

    // see if highlighting is required
    bool highlight(
        (!attributes.hasFlag( TaskAttributes::IsSeparator )) &&
        (XmlOptions::get().get<bool>("HIGHLIGHT_ON_ENTER") || XmlOptions::get().get<bool>("HIGHLIGHT_ON_CLICK") ) );

    // highlight intensity
    maxHighlightIndex_ = highlight ? XmlOptions::get().get<int>( "HIGHLIGHT_STEPS" ):1;

    // reset highlight properties
    fullIntensity_ = XmlOptions::get().get<double>( "HIGHLIGHT_INTENSITY" );

    // allocate pixmap vector
    pixmaps_ = List( maxSizeIndex_*maxHighlightIndex_ );

}

//_________________________________________________________________________
PixmapList::~PixmapList(void)
{}

//_________________________________________________________________________
QPixmap PixmapList::get( int sizeIndex, int highlightIndex ) const
{

    int index( _index( sizeIndex, highlightIndex ) );
    if( !pixmaps_[index].isNull() ) return pixmaps_[index];

    // see if base (un-highlighted) pixmap is already created
    // create it otherwise
    int baseIndex( _index( sizeIndex, 0 ) );
    CustomPixmap base( pixmaps_[baseIndex] );
    if( base.isNull() )
    {

        // separator pixmap
        if( attributes_.hasFlag( TaskAttributes::IsSeparator ) )
        {

            const TaskBarAttributes::Layout& layout( taskBarAttributes_.layout() );
            if( layout & TaskBarAttributes::Horizontal )
            {

                base = SeparatorPixmap( sizeIndex*sizeIncrement_+4, taskBarAttributes_.hasFlag( TaskBarAttributes::ExpandSeparators ) ? (sizeIndex*sizeIncrement_ + minSize_):minSize_, layout );

            } else if( layout & TaskBarAttributes::Vertical ) {

                base = SeparatorPixmap( taskBarAttributes_.hasFlag( TaskBarAttributes::ExpandSeparators ) ? (sizeIndex*sizeIncrement_+minSize_):minSize_, sizeIndex*sizeIncrement_ + 4, layout );

            } else {

                base  = SeparatorPixmap( sizeIndex*sizeIncrement_ + 4, sizeIndex*sizeIncrement_ + 4, layout );

            }

        } else {

            // pixmap from icon
            const QPixmap& pixmap( PixmapEngine::get( attributes_.iconName() ) );
            Q_ASSERT( !pixmap.isNull() );

            //if( pixmap.isNull() ) pixmap = PixmapEngine::get( ICONS::DEFAULT );

            // scaled pixmap
            base = pixmap.scaled( QSize( sizeIndex*sizeIncrement_ + minSize_, sizeIndex*sizeIncrement_ + minSize_ ), Qt::KeepAspectRatio, Qt::SmoothTransformation );

        }

        // store as base (un-highlighted) pixmap
        if( baseIndex != index )
        { const_cast<List&>(pixmaps_)[baseIndex] = base; }

    }

    // highlight
    if( !attributes_.hasFlag( TaskAttributes::IsSeparator ) && highlightIndex && maxHighlightIndex_ > 1 )
    { base = base.highlighted( fullIntensity_*(highlightIndex+1)/( maxHighlightIndex_ )  ); }

    return ( const_cast<List&>(pixmaps_)[index] = base );

}

//_________________________________________________________________________
int PixmapList::_index( int sizeIndex, int highlightIndex ) const
{ return qMin( sizeIndex, maxSizeIndex_ - 1) + maxSizeIndex_*qMin( highlightIndex, maxHighlightIndex_-1 ); }
