// $Id: TaskAttributesDelegate.cpp,v 1.1 2013/02/13 10:28:57 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*******************************************************************************/

#include "TaskAttributesDelegate.h"

#include "PixmapEngine.h"
#include "TaskAttributesModel.h"
#include "TaskAttributes.h"

#include <QtGui/QAbstractTextDocumentLayout>
#include <QtGui/QApplication>
#include <QtGui/QStyleOptionViewItem>
#include <QtGui/QPainter>

//_________________________________________________________
TaskAttributesDelegate::TaskAttributesDelegate( QObject *parent ):
    QAbstractItemDelegate( parent ),
    Counter( "Delegate" ),
    margin_( 4 ),
    iconSize_( 32 )
{ Debug::Throw( "TaskAttributesDelegate::TaskAttributesDelegate.\n" ); }

//_________________________________________________________
void TaskAttributesDelegate::paint( QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index ) const
{

    // check index
    if ( !index.isValid() ) return;

    // cast model
    const TaskAttributesModel& model( *static_cast<const TaskAttributesModel*>( index.model() ) );
    if( !model.contains( index ) ) return;

    // selection
    QStyleOptionViewItemV4 viewOption(option);
    viewOption.showDecorationSelected = true;
    QStyle *style = viewOption.widget ? viewOption.widget->style() : QApplication::style();
    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &viewOption, painter, viewOption.widget);

    // get task attributes
    const TaskAttributes& attributes( model.get( index ) );
    if( attributes.hasFlag( TaskAttributes::IsSeparator ) )
    {

        // draw a separator
        QStyleOptionFrameV3 frameOption;
        frameOption.rect = option.rect;
        frameOption.frameShape = QFrame::HLine;

        QApplication::style()->drawControl( QStyle::CE_ShapedFrame, &frameOption, painter, 0x0 );

    } else {
        // create layout
        QTextDocument document;
        document.setDefaultFont( option.font );
        QTextCursor cursor( &document );

        // modify color
        QTextCharFormat charFormat( cursor.charFormat() );
        QPalette::ColorGroup colorGroup = option.state & QStyle::State_Enabled ? QPalette::Normal : QPalette::Disabled;
        if( colorGroup == QPalette::Normal && !(option.state & QStyle::State_Active) )
        { colorGroup = QPalette::Inactive; }

        QColor foreground;
        if( option.state & QStyle::State_Selected ) foreground = option.palette.color( colorGroup, QPalette::HighlightedText );
        else foreground = option.palette.color( colorGroup, QPalette::Text );

        charFormat.setForeground( foreground );
        cursor.setCharFormat( charFormat );
        _layoutText( cursor, attributes );

        // draw icon
        QPixmap pixmap( PixmapEngine::get( attributes.iconName() ) );
        if( !pixmap.isNull() )
        {
            pixmap = pixmap.scaled( QSize( iconSize_, iconSize_ ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
            painter->drawPixmap( option.rect.topLeft()+QPoint( margin_, margin_ ), pixmap );
        }

        // draw text
        painter->save();
        painter->translate( option.rect.topLeft() + QPoint( iconSize_ + 2*margin_, 0 ) );
        document.drawContents( painter );
        painter->restore();

    }

    // draw focus
    _drawFocus( painter, option, option.rect );

}

//_________________________________________________________
QSize TaskAttributesDelegate::sizeHint( const QStyleOptionViewItem &option, const QModelIndex &index ) const
{

    if ( !index.isValid() ) return QSize();

    // cast model
    const TaskAttributesModel& model( *static_cast<const TaskAttributesModel*>( index.model() ) );
    if( !model.contains( index ) ) return QSize();

    // get task attributes
    const TaskAttributes& attributes( model.get( index ) );
    if( attributes.hasFlag( TaskAttributes::IsSeparator ) ) return QSize( 4*margin_, 4*margin_ );

    // icon rect
    QSize iconRect( iconSize_+2*margin_, iconSize_+2*margin_ );

    // create document and layout text
    QTextDocument document;
    document.setDefaultFont( option.font );
    QTextCursor cursor( &document );
    _layoutText( cursor, attributes );

    QSize textRect( document.documentLayout()->frameBoundingRect( document.rootFrame() ).size().toSize() );

    // size
    return QSize( iconRect.width() + textRect.width(), qMax( iconRect.height(), textRect.height() ) );

}

//_________________________________________________________
void TaskAttributesDelegate::_layoutText( QTextCursor& cursor, const TaskAttributes& attributes ) const
{

    QTextCharFormat defaultFormat( cursor.charFormat() );
    QTextCharFormat boldFormat( defaultFormat );
    boldFormat.setFontWeight( QFont::Bold );
    cursor.insertText( attributes.name()+"\n", boldFormat );
    cursor.insertText( attributes.comments()+"\n", defaultFormat );
    cursor.insertText( attributes.command(), defaultFormat );

}

//_________________________________________________________
void TaskAttributesDelegate::_drawFocus( QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect ) const
{
    if (option.state & QStyle::State_HasFocus)
    {

        QStyleOptionFocusRect o;
        o.QStyleOption::operator=(option);
        o.rect = rect;
        o.state |= QStyle::State_KeyboardFocusChange;
        QPalette::ColorGroup colorGroup = (option.state & QStyle::State_Enabled) ? QPalette::Normal : QPalette::Disabled;
        o.backgroundColor = option.palette.color( colorGroup, (option.state & QStyle::State_Selected) ? QPalette::Highlight : QPalette::Background );

        QApplication::style()->drawPrimitive( QStyle::PE_FrameFocusRect, &o, painter );

    }

}
