// $Id: TaskBarAttributes.cpp,v 1.31 2013/02/13 10:28:57 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*******************************************************************************/

#include "TaskBarAttributes.h"
#include "XmlDef.h"
#include "Str.h"

//_____________________________________
unsigned int& TaskBarAttributes::_idCount( void )
{
    static unsigned int counter = 0;
    return counter;
}

//_____________________________________
TaskBarAttributes::ModeNameMap& TaskBarAttributes::_modeNames( void )
{
    static TaskBarAttributes::ModeNameMap names;
    return names;
}

//________________________________________________________________________
TaskBarAttributes::LayoutNameMap& TaskBarAttributes::_layoutNames( void )
{
    static TaskBarAttributes::LayoutNameMap names;
    return names;
}

//________________________________________________________________________
TaskBarAttributes::PositionNameMap& TaskBarAttributes::_positionNames( void )
{
    static TaskBarAttributes::PositionNameMap names;
    return names;
}

//________________________________________________________________________
void TaskBarAttributes::_initializeModeNames( void )
{
    Debug::Throw( "TaskBarAttributes::_initializeModeNames.\n" );
    Q_ASSERT( _modeNames().empty() );
    _modeNames().insert( Fixed, "Fixed" );
    _modeNames().insert( Minimum, "Minimum" );
    _modeNames().insert( Parabolic, "Parabolic" );
}

//________________________________________________________________________
void TaskBarAttributes::_initializeLayoutNames( void )
{
    Debug::Throw( "TaskBarAttributes::_initializeLayoutNames.\n" );
    Q_ASSERT( _layoutNames().empty() );
    _layoutNames().insert( HorizontalTop, "Horizontal Top" );
    _layoutNames().insert( HorizontalCenter, "Horizontal Centered" );
    _layoutNames().insert( HorizontalBottom, "Horizontal Bottom" );

    _layoutNames().insert( VerticalLeft, "Vertical Left" );
    _layoutNames().insert( VerticalCenter, "Vertical Centered" );
    _layoutNames().insert( VerticalRight, "Vertical Right" );

    _layoutNames().insert( Box, "Box" );
}

//________________________________________________________________________
void TaskBarAttributes::_initializePositionNames( void )
{
    Debug::Throw( "TaskBarAttributes::_initializePositionNames.\n" );
    _positionNames().insert( PositionAny, "Any" );
    _positionNames().insert( PositionTopLeft, "Top Left" );
    _positionNames().insert( PositionTopCenter, "Top Center" );
    _positionNames().insert( PositionTopRight, "Top Right" );

    _positionNames().insert( PositionBottomLeft, "Bottom Left" );
    _positionNames().insert( PositionBottomCenter, "Bottom Center" );
    _positionNames().insert( PositionBottomRight, "Bottom Right" );

    _positionNames().insert( PositionLeftCenter, "Center Left" );
    _positionNames().insert( PositionRightCenter, "Center Right" );

}

//____________________________________________________________________________________
QString TaskBarAttributes::positionName( PositionMode position )
{
    Debug::Throw() << "TaskBarAttributes::positionName - " << position << endl;
    PositionNameMap::const_iterator iter( positionNames().find( position ) );
    Q_ASSERT( iter != positionNames().end() );
    return iter.value();
}

//___________________________________________
TaskBarAttributes::TaskBarAttributes( const QDomElement& element ):
    Counter( "TaskBarAttributes" ),
    id_( _idCount()++ ),
    name_( "generic taskbar" ),
    layout_( HorizontalCenter ),
    mode_( Parabolic ),
    nColumns_( 2 ),
    positionMode_( PositionAny ),
    position_( 0, 0 ),
    zoomDelay_( 1 ),
    flags_(0),
    scale_( 5 ),
    needTaskUpdate_( false )
{
    Debug::Throw( "TaskBarAttributes::TaskBarAttributes (DOM).\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( unsigned int i=0; i<attributes.length(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name == XML::NAME ) setName( value );
        else if( name == XML::LAYOUT ) setLayout( value );
        else if( name == XML::MODE ) setMode( value );
        else if( name == XML::COLUMNS ) setNumColumns( value.toInt() );
        else if( name == XML::SCALE ) setScale( value.toDouble() );
        else if( name == XML::POSITION ) setPositionMode( value );
        else if( name == XML::X ) setX( value.toInt() );
        else if( name == XML::Y ) setY( value.toInt() );
        else if( name == XML::ZOOM_DELAY ) setZoomDelay( value.toInt() );
        else if( name == XML::FLAGS ) setFlags( Flags( value.toInt() ) );
        else Debug::Throw(0) << "TaskBarAttributes::TaskBarAttributes - unrecognized attribute " << name << endl;
    }

    // parse children elements
    for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
    {
        QDomElement childElement = child_node.toElement();
        if( childElement.isNull() ) continue;

        QString tag_name( childElement.tagName() );
        if( tag_name == XML::TASK ) addTaskAttributes( TaskAttributes( childElement ) );
        else Debug::Throw(0) << "TaskBarAttributes::TaskBarAttributes - unrecognized child " << tag_name << endl;
    }

}

//____________________________________________________________________________________
QDomElement TaskBarAttributes::domElement( QDomDocument& parent ) const
{
    Debug::Throw( "TaskBarAttributes::DomElement.\n" );

    QDomElement out( parent.createElement( XML::TASKBAR ) );
    if( !name().isEmpty() ) out.setAttribute( XML::NAME, name() );
    out.setAttribute( XML::LAYOUT, layoutName() );
    out.setAttribute( XML::MODE, modeName() );
    out.setAttribute( XML::COLUMNS, Str().assign<unsigned int>( numColumns() ) );
    out.setAttribute( XML::SCALE, Str().assign<double>( scale() ) );
    out.setAttribute( XML::POSITION, positionName() );
    out.setAttribute( XML::X, Str().assign<int>(position_.x()) );
    out.setAttribute( XML::Y, Str().assign<int>(position_.y()) );
    out.setAttribute( XML::ZOOM_DELAY, Str().assign<unsigned int>( zoomDelay() ) );
    out.setAttribute( XML::FLAGS, Str().assign<unsigned int>( flags() ) );

    // tasks
    foreach( const TaskAttributes& attributes, taskList_ )
    { out.appendChild( attributes.domElement( parent ) ); }

    return out;

}

//____________________________________________________________________________________
bool TaskBarAttributes::differs( const TaskBarAttributes& other ) const
{
    Debug::Throw( "TaskBarAttributes::differs.\n" );
    if( name() != other.name() ) return true;
    else if( layout() != other.layout() ) return true;
    else if( mode() != other.mode() ) return true;
    else if( numColumns() != other.numColumns() ) return true;
    else if( zoomDelay() != other.zoomDelay() ) return true;
    else if( positionMode() != other.positionMode() ) return true;
    else if( scale() != other.scale() ) return true;
    else if( flags() != other.flags() ) return true;
    else if( taskAttributesList().differs( other.taskAttributesList() ) ) return true;
    else return false;
}

//____________________________________________________________________________________
bool TaskBarAttributes::needTaskUpdate( const TaskBarAttributes& other )
{
    Debug::Throw( "TaskBarAttributes::needTaskUpdate.\n" );

    if( needTaskUpdate_ )
    {
        needTaskUpdate_ = false;
        return true;
    }

    // check task list
    else if( taskAttributesList() != other.taskAttributesList() ) return true;

    // check flags
    else if( layout() != other.layout() ) return true;
    else if( hasFlag( ExpandSeparators ) != other.hasFlag( ExpandSeparators ) ) return true;
    else if( hasFlag( ShowName ) != other.hasFlag( ShowName ) ) return true;
    else if( hasFlag( ZoomEnabled ) != other.hasFlag( ZoomEnabled ) ) return true;
    else return false;
}

//____________________________________________________________________________________
void TaskBarAttributes::setLayout( const QString& layout )
{

    for( LayoutNameMap::const_iterator iter = layoutNames().begin(); iter != layoutNames().end(); ++iter )
    {
        if( layout.toLower() == iter.value().toLower() )
        {
            setLayout( iter.key() );
            return;
        }
    }

}

//____________________________________________________________________________________
void TaskBarAttributes::setMode( const QString& mode )
{

    for( ModeNameMap::const_iterator iter = modeNames().begin(); iter != modeNames().end(); ++iter )
    {
        if( mode.toLower() == iter.value().toLower() )
        {
            setMode( iter.key() );
            return;
        }
    }

}

//____________________________________________________________________________________
void TaskBarAttributes::setPositionMode( const QString& position )
{
    for( PositionNameMap::const_iterator iter = positionNames().begin(); iter != positionNames().end(); ++iter )
    {
        if( position == iter.value() )
        {
            setPositionMode( iter.key() );
            return;
        }
    }
}

//___________________________________________
bool TaskBarAttributes::List::differs( const QList<TaskBarAttributes>& other ) const
{
    if( size() != other.size() ) return true;
    else {

        for( int index = 0; index < size(); ++index )
        { if( at(index).differs( other.at(index) ) ) return true; }

        return false;
    }

}
