
// $Id: TaskBarConfiguration.cpp,v 1.37 2013/02/13 10:28:57 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*******************************************************************************/

#include "TaskBarConfiguration.h"
#include "GridLayout.h"
#include "AnimatedLineEditor.h"
#include "CustomSlider.h"
#include "TextEditor.h"
#include "Debug.h"
#include "XmlOptions.h"
#include "QtUtil.h"

#include <QtGui/QLabel>
#include <QtGui/QGroupBox>
#include <QtGui/QLayout>
#include <QtGui/QButtonGroup>

//______________________________________________
TaskBarConfiguration::TaskBarConfiguration( QWidget* parent, const TaskBarAttributes& attributes ):
    QWidget( parent ),
    Counter( "TaskBarConfiguration" )
{
    Debug::Throw( "TaskBarConfiguration::TaskBarConfiguration.\n" );

    // layout
    setLayout( new QHBoxLayout() );
    layout()->setSpacing(10);
    layout()->setMargin(5);

    QVBoxLayout* vLayout = new QVBoxLayout();
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    layout()->addItem( vLayout );

    // name
    QGroupBox *box;
    GridLayout *gridLayout;
    vLayout->addWidget( box = new QGroupBox( "Name", this ));
    box->setLayout( new QVBoxLayout() );
    box->layout()->setSpacing(5);
    box->layout()->setMargin(5);

    box->layout()->addWidget( nameLineEdit_ = new AnimatedLineEditor( box ) );
    nameLineEdit_->setToolTip( "Taskbar name" );

    // layout
    vLayout->addWidget( box = new QGroupBox( "Layout", this ) );
    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    box->setLayout( gridLayout );

    gridLayout->addWidget( new QLabel( "Layout/alignment:", box ) );
    gridLayout->addWidget( layoutComboBox_ = new QComboBox( box ) );

    const TaskBarAttributes::LayoutNameMap& layout_names( TaskBarAttributes::layoutNames() );
    for( TaskBarAttributes::LayoutNameMap::const_iterator iter = layout_names.begin(); iter != layout_names.end(); ++iter )
    { layoutComboBox_->addItem( iter.value() ); }

    layoutComboBox_->setToolTip(
        "Choose taskbar layout and button \n"
        "alignment within taskbar." );

    connect( layoutComboBox_, SIGNAL( activated( const QString& ) ), this, SLOT( _updateLayout( const QString& ) ) );

    // zoom
    gridLayout->addWidget( new QLabel( "Zoom mode:", box ) );
    gridLayout->addWidget( modeComboBox_ = new QComboBox( box ) );
    const TaskBarAttributes::ModeNameMap& mode_names( TaskBarAttributes::modeNames() );
    for( TaskBarAttributes::ModeNameMap::const_iterator iter = mode_names.begin(); iter != mode_names.end(); ++iter )
    { modeComboBox_->addItem( iter.value() ); }
    modeComboBox_->setToolTip( "Taskbar zoom mode.\n" );

    connect( modeComboBox_, SIGNAL( activated( const QString& ) ), this, SLOT( _updateMode( const QString& ) ) );

    gridLayout->addWidget( new QLabel( "Number of columns: ", box ) );
    gridLayout->addWidget( columnSpinBox_ = new QSpinBox( box ) );
    columnSpinBox_->setMinimum( 2 );
    columnSpinBox_->setMaximum( 20 );
    columnSpinBox_->setToolTip( "Number of columns for the box layout" );

    gridLayout->addWidget( new QLabel( "Zoom scale: ", box ) );
    gridLayout->addWidget( scaleSpinBox_ = new QSpinBox( box ) );
    scaleSpinBox_->setMinimum( 1 );
    scaleSpinBox_->setMaximum( 100 );
    scaleSpinBox_->setToolTip( "zoom" );

    QCheckBox* checkbox;
    gridLayout->addWidget( checkbox = new QCheckBox( "zoom delay (milliseconds)", box ) );
    checkbox->setToolTip( "Enable/disable zoom delay.\n" );
    checkboxes_.insert( TaskBarAttributes::ZoomEnabled, checkbox );

    gridLayout->addWidget( delaySpinBox_ = new QSpinBox( box ) );
    delaySpinBox_->setMinimum( 0 );
    delaySpinBox_->setMaximum( 1000 );
    delaySpinBox_->setToolTip( "Interval between two updates while zooming" );

    // by default, delay spinbox is disabled, since the corresponding checkbox is not checked
    delaySpinBox_->setEnabled( false );

    connect( checkbox, SIGNAL( toggled( bool ) ), delaySpinBox_, SLOT( setEnabled( bool ) ) );

    // position box
    vLayout->addWidget( _positionBox( this ) );

    // flags
    layout()->addWidget( box = new QGroupBox( "Flags", this ) );
    vLayout = new QVBoxLayout();
    vLayout->setSpacing( 5 );
    vLayout->setMargin(5);
    box->setLayout( vLayout );

    vLayout->addWidget( checkbox = new QCheckBox( "Lock position", box ) );
    checkbox->setToolTip(
        "Lock taskbar position on screen.\n"
        "Note: this option overrides the\n "
        "global configuration settings.\n" );
    checkboxes_.insert( TaskBarAttributes::LockPosition, checkbox );

    vLayout->addWidget( checkbox = new QCheckBox( "Auto hide", box ) );
    checkbox->setToolTip( "Hide taskbar when unused.\n" );
    checkboxes_.insert( TaskBarAttributes::AutoHide, checkbox );

    vLayout->addWidget( checkbox = new QCheckBox( "Stays on top", box ) );
    checkbox->setToolTip( "Controls if taskbar must stay on top of all windows\n");
    checkboxes_.insert( TaskBarAttributes::StaysOnTop, checkbox );

    vLayout->addWidget( checkbox =  new QCheckBox( "Sticky", box ) );
    checkbox->setToolTip( "Make window appear on all desktops" );
    checkboxes_.insert( TaskBarAttributes::Sticky, checkbox );

    vLayout->addWidget( checkbox =  new QCheckBox( "Hide from taskbar", box ) );
    checkbox->setToolTip( "Hide window from taskbar" );
    checkboxes_.insert( TaskBarAttributes::HideFromTaskBar, checkbox );

    vLayout->addWidget( checkbox = new QCheckBox( "Round corners", box ) );
    checkbox->setToolTip( "Round taskbar corners\n");
    checkboxes_.insert( TaskBarAttributes::RoundCorners, checkbox );

    QFrame* frame;
    vLayout->addWidget( frame = new QFrame( this ) );
    frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    // button flags
    vLayout->addWidget( checkbox = new QCheckBox( "Lock buttons position", box ) );
    checkbox->setToolTip(
        "Lock buttons position in taskbar.\n"
        "Note: this option overrides the\n "
        "global configuration settings.\n" );
    checkboxes_.insert( TaskBarAttributes::LockButtons, checkbox );

    vLayout->addWidget( checkbox = new QCheckBox( "Show button names", box ) );
    checkbox->setToolTip(
        "Show button names in taskbar.\n"
        "Note: this option overrides the\n "
        "global configuration settings.\n"
        "It gets overriden by individual\n"
        "button settings" );
    checkboxes_.insert( TaskBarAttributes::ShowName, checkbox );

    vLayout->addWidget( checkbox = new QCheckBox( "expand separators", box ) );
    checkbox->setToolTip(
        "Controls if separators are expanded\n"
        "while zooming in OsX mode\n "
        ".\n" );
    checkboxes_.insert( TaskBarAttributes::ExpandSeparators, checkbox );

    for( CheckBoxMap::iterator iter = checkboxes_.begin(); iter != checkboxes_.end(); ++iter )
    { connect( iter.value(), SIGNAL( toggled( bool ) ), SLOT( _updateCheckBoxes() ) ); }

    vLayout->addStretch(1);

    setAttributes( attributes );

}

//_______________________________________
void TaskBarConfiguration::setAttributes( const TaskBarAttributes& attributes )
{
    Debug::Throw( "TaskBarConfiguration::SetAttributes.\n" );

    attributes_ = attributes;
    backupAttributes_ = attributes;

    nameLineEdit_->setText( attributes.name() );
    layoutComboBox_->setCurrentIndex( layoutComboBox_->findText( attributes.layoutName() ) );
    modeComboBox_->setCurrentIndex( modeComboBox_->findText( attributes.modeName() ) );
    delaySpinBox_->setValue( attributes.zoomDelay() );
    columnSpinBox_->setValue( attributes.numColumns() );
    scaleSpinBox_->setValue( int( attributes.scale()*10 ) );

    for( CheckBoxMap::iterator iter = checkboxes_.begin(); iter != checkboxes_.end(); ++iter )
    { iter.value()->setChecked( attributes.hasFlag( iter.key() ) ); }

    if( attributes.positionMode() != TaskBarAttributes::PositionAny )
    {
        for( ButtonMap::iterator iter = positionButtons_.begin(); iter != positionButtons_.end(); ++iter )
        { if( iter.value() == attributes.positionMode() ) iter.key()->setChecked( true ); }
    }

    _updateLayout( attributes.layoutName() );
    _updateMode( attributes.modeName() );

}
//_______________________________________
const TaskBarAttributes& TaskBarConfiguration::attributes( void )
{

    Debug::Throw("TaskBarConfiguration::Attributes.\n");

    attributes_.setName( nameLineEdit_->text() );
    attributes_.setLayout( layoutComboBox_->currentText() );
    attributes_.setMode( modeComboBox_->currentText() );
    attributes_.setZoomDelay( delaySpinBox_->value() );
    attributes_.setNumColumns( columnSpinBox_->value() );
    attributes_.setScale( double(scaleSpinBox_->value())/10 );

    for( CheckBoxMap::iterator iter = checkboxes_.begin(); iter != checkboxes_.end(); ++iter )
    { attributes_.setFlag( iter.key(), iter.value()->isChecked() ); }

    // set prefered position
    TaskBarAttributes::PositionMode position( TaskBarAttributes::PositionAny );
    for( ButtonMap::iterator iter = positionButtons_.begin(); iter != positionButtons_.end(); ++iter )
    { if( iter.key()->isChecked() )  position = iter.value(); }

    attributes_.setPositionMode( position );

    // check for incompatibilities in switches
    return attributes_;

}

//_______________________________________________
void TaskBarConfiguration::_togglePosition( QAbstractButton* button )
{

    Debug::Throw( "TaskBarConfiguration::_togglePosition" );
    ButtonMap::iterator clicked_iter( positionButtons_.find( button ) );
    Q_ASSERT( clicked_iter != positionButtons_.end() );
    if( !clicked_iter.key()->isChecked() ) return;

    for( ButtonMap::iterator iter = positionButtons_.begin(); iter != positionButtons_.end(); ++iter )
    { if( iter.key() != clicked_iter.key() ) iter.key()->setChecked( false ); }

    // change checkboxes enability
    _updateCheckBoxes();

}

//_______________________________________________
void TaskBarConfiguration::_updateLayout( const QString& value )
{
    columnSpinBox_->setEnabled( value == TaskBarAttributes::layoutName( TaskBarAttributes::Box ) );
    _updateCheckBoxes();
}

//_______________________________________________
void TaskBarConfiguration::_updateMode( const QString& value )
{ scaleSpinBox_->setEnabled( value == TaskBarAttributes::modeName( TaskBarAttributes::Parabolic ) ); }

//_______________________________________________
QWidget* TaskBarConfiguration::_positionBox( QWidget* parent )
{
    Debug::Throw( "TaskBarConfiguration::_positionBox.\n" );
    QGroupBox* box = new QGroupBox( "Prefered Position", parent );
    box->setLayout( new QVBoxLayout() );
    box->layout()->setSpacing(5);
    box->layout()->setMargin(5);

    QGridLayout *gridLayout = new QGridLayout();
    gridLayout->setSpacing(10);
    gridLayout->setMargin(0);
    box->layout()->addItem( gridLayout );

    // needed to match button to position mode
    static TaskBarAttributes::Position positions[3] =
    {
        TaskBarAttributes::PositionLeft,
        TaskBarAttributes::PositionCenter,
        TaskBarAttributes::PositionRight
    };

    QButtonGroup* button_group =  new QButtonGroup( box );
    button_group->setExclusive( false );
    QPushButton *button;
    for( unsigned int i=0; i<3; i++ )
    {
        // top buttons
        TaskBarAttributes::PositionMode position = TaskBarAttributes::PositionTop | positions[i];
        button_group->addButton( button = new QPushButton( box ) );
        button->setToolTip( TaskBarAttributes::positionName( position ) );
        gridLayout->addWidget( button, 0, i );
        positionButtons_.insert( button, position );

        // bottom buttons
        position = TaskBarAttributes::PositionBottom | positions[i];
        button_group->addButton( button = new QPushButton( box ) );
        button->setToolTip( TaskBarAttributes::positionName( position ) );
        gridLayout->addWidget( button, 2, i );
        positionButtons_.insert( button, position );

    }

    // left centered
    button_group->addButton( button = new QPushButton( box ) );
    button->setToolTip( TaskBarAttributes::positionName( TaskBarAttributes::PositionLeftCenter ) );
    gridLayout->addWidget( button, 1, 0 );
    positionButtons_.insert( button, TaskBarAttributes::PositionLeftCenter );

    // right centered
    button_group->addButton( button = new QPushButton( box ) );
    button->setToolTip( TaskBarAttributes::positionName( TaskBarAttributes::PositionRightCenter ) );
    gridLayout->addWidget( button, 1, 2 );
    positionButtons_.insert( button, TaskBarAttributes::PositionRightCenter );

    // set size and toggle
    for( ButtonMap::iterator iter = positionButtons_.begin(); iter != positionButtons_.end(); ++iter )
    { iter.key()->setCheckable( true ); }

    // set signal
    connect( button_group, SIGNAL( buttonClicked( QAbstractButton* ) ), SLOT( _togglePosition( QAbstractButton* ) ) );
    return box;

}

//_______________________________________________
void TaskBarConfiguration::_updateCheckBoxes( void )
{
    { Debug::Throw("TaskBarConfiguration::_updateCheckBoxes.\n"); }

    attributes();

    // AutoHide is enabled only if and layout match with position.
    checkboxes_[TaskBarAttributes::AutoHide ]->setEnabled(
        ( ( attributes_.positionMode() == TaskBarAttributes::PositionBottomCenter ||
        attributes_.positionMode() == TaskBarAttributes::PositionBottomLeft ||
        attributes_.positionMode() == TaskBarAttributes::PositionBottomRight ) &&
        attributes_.layout() ==  TaskBarAttributes::HorizontalBottom ) ||

        ( ( attributes_.positionMode() == TaskBarAttributes::PositionTopCenter ||
        attributes_.positionMode() == TaskBarAttributes::PositionTopLeft ||
        attributes_.positionMode() == TaskBarAttributes::PositionTopRight ) &&
        attributes_.layout() ==  TaskBarAttributes::HorizontalTop ) ||

        ( ( attributes_.positionMode() == TaskBarAttributes::PositionTopLeft ||
        attributes_.positionMode() == TaskBarAttributes::PositionBottomLeft ||
        attributes_.positionMode() == TaskBarAttributes::PositionLeftCenter ) &&
        attributes_.layout() ==  TaskBarAttributes::VerticalLeft ) ||

        ( ( attributes_.positionMode() == TaskBarAttributes::PositionTopRight ||
        attributes_.positionMode() == TaskBarAttributes::PositionBottomRight ||
        attributes_.positionMode() == TaskBarAttributes::PositionRightCenter ) &&
        attributes_.layout() ==  TaskBarAttributes::VerticalRight ) );


}
