
// $Id: TaskButton.cpp,v 1.38 2013/02/13 10:28:57 hupereir Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Debug.h"
#include "Effects.h"
#include "Options.h"
#include "SeparatorPixmap.h"
#include "TaskButton.h"
#include "TransparentWidget.h"
#include "XmlOptions.h"

#include <QtGui/QApplication>
#include <QtGui/QFont>
#include <cmath>

// this class debug level
static const int debugLevel = 1;

//_____________________________________________
void TaskButton::setAttributes( const TaskBarAttributes& bar_attributes, const TaskAttributes& attributes )
{

    Debug::Throw() << "TaskButton::SetAttributes - " << attributes.name() << endl;

    // store initial max size (for the need_resize flag)
    QSize old_size( maximumSize() );

    // store task
    attributes_ = attributes;

    // highlight steps
    maximumHighlightIndex_ = XmlOptions::get().get<unsigned int>( "HIGHLIGHT_STEPS" ) + 1;

    // pixmap max/max size
    unsigned int max_size( XmlOptions::get().get<unsigned int>( "MAX_BUTTON_SIZE" ) );

    //! clear pixmap list
    _pixmaps().setAttributes( bar_attributes, attributes );

    // for separators, only create one pixmap
    if( attributes.hasFlag( TaskAttributes::IsSeparator ) ) showName_ = false;
    else {

        // add needed space for text
        unsigned int width = max_size;
        unsigned int height = max_size;
        showName_ = bar_attributes.hasFlag( TaskBarAttributes::ShowName ) || attributes_.hasFlag( TaskAttributes::ShowName );
        if( showName_ )
        {

            TRANSPARENCY::TransparentWidget widget;

            // retrieve options
            shadowColor_ = widget.shadowColor();
            shadowOffset_ = widget.shadowOffset();
            foregroundColor_ = widget.foregroundColor();

            QFontMetrics font( qApp->fontMetrics() );
            textSize_ = font.size( 0, attributes_.name() );
            width = qMax<int>( width, textSize_.width() );
            height += textSize_.height();

        }
    }

    // validity
    /* button must be a separator, or its pixmap must be valid, or its name must be valid and shown */
    _setIsValid(
        attributes.hasFlag( TaskAttributes::IsSeparator ) ||
        !CustomPixmap( attributes.iconName() ).isNull() ||
        ( ( bar_attributes.hasFlag( TaskBarAttributes::ShowName ) ||
        attributes_.hasFlag( TaskAttributes::ShowName ) ) &&
        !attributes.name().isEmpty() ) );

    // set button maximum size
    _setNeedResize( old_size != maximumSize() );
    return;
}

//___________________________________________________
QSize TaskButton::size( const unsigned int& index) const
{

    // check pixmaps
    if( !_pixmaps().maxSizeIndex() ) return QSize();

    // size
    if( attributes_.hasFlag( TaskAttributes::IsSeparator ) ) return _pixmaps().get( index, 0 ).size();

    // return size (adding text size if needed)
    return QSize(
        qMax( textSize_.width(), _pixmaps().get( index, 0 ).width() ),
        textSize_.height()+ _pixmaps().get( index, 0 ).height() );

}

//__________________________________________________________
void TaskButton::setDirection( const TaskButton::Direction& value )
{
    Debug::Throw( "TaskButton::setDirection\n" );
    if( value == direction_ ) return;

    // update value
    direction_ = value;

}

//__________________________________________________________
const TaskButton::HighlightDirection& TaskButton::updateHighlightDirection( void )
{

    if( highlightDirection_ == Lighter )
    {

        if( highlightIndex_ < maximumHighlightIndex_-1 )
        {

            highlightIndex_+=0.1;
            _updateTrueHighlightIndex();

        } else highlightDirection_ = Darker;

    } else if( highlightDirection_ == Darker ) {

        if( highlightIndex_ > 0 ) {

            highlightIndex_ -= 0.1;
            _updateTrueHighlightIndex();

        } else highlightDirection_ = (highlight_) ? Lighter:NoHighlight;

    }

    return highlightDirection_;

}

//__________________________________________________________
const TaskButton::Direction& TaskButton::updateDirection( void )
{
    if( direction_ == Bigger && index_ < _pixmaps().maxSizeIndex() -1 ) index_++;
    else if( direction_ == Smaller && index_ > 0 ) index_--;
    else direction_ = NoDirection;
    return direction_;
}

//___________________________________________________
void TaskButton::paint( QPainter& painter ) const
{

    Debug::Throw( "TaskButton::Paint.\n" );
    _drawPixmap( painter, index_ );
    if( showName_ ) _drawText( painter, index_ );

}

//___________________________________________________
void TaskButton::_drawPixmap( QPainter& painter, const unsigned int& index ) const
{

    Debug::Throw( "TaskButton::_drawPixmap .\n" );

    // store pointer to pixmap to be drawn
    const QPixmap& pixmap( _pixmaps().get( index, attributes_.hasFlag( TaskAttributes::IsSeparator ) ? 0:highlightIndex() ) );

    // center pixmap
    int h_offset( ( rect_.width() - pixmap.width() )/2 );
    int v_offset( ( rect_.height() - pixmap.height() )/2 );
    if( showName_ ) v_offset -= textSize_.height()/2;

    // paint
    painter.drawPixmap( rect_.x() + h_offset, rect_.y() + v_offset, pixmap );

}

//___________________________________________________
void TaskButton::_drawText( QPainter& painter, const unsigned int& index ) const
{

    if( !foregroundColor_.isValid() ) return;

    // this should go to the cache too. To speed things up
    // retrieve pixmap to assign text position
    QPixmap tmp( rect_.size() );
    tmp.fill( Qt::transparent );
    {

        QPainter painter( &tmp );
        const QPixmap& pixmap( _pixmaps().get( index, 0 ) );
        int y = ( rect_.height() + pixmap.height() - textSize_.height() )/2;

        painter.setPen( foregroundColor_ );
        painter.drawText(
            tmp.rect().left(),
            y,
            tmp.width(),
            textSize_.height(),
            Qt::AlignCenter,
            attributes_.name() );

    }

    // blur
    if( shadowOffset_ > 0 && shadowColor_.isValid() )
    {
        QImage image( tmp.toImage() );
        TRANSPARENCY::Effects::shadowBlur(image, shadowOffset_, shadowColor_ );
        painter.drawImage( rect_.topLeft(), image);
    }

    painter.drawPixmap( rect_.topLeft(), tmp );

}

//_______________________________________________________________________________
void TaskButton::_updateTrueHighlightIndex( void )
{ trueHighlightIndex_ = (unsigned int)( ( cos( M_PI*(1-highlightIndex_/(maximumHighlightIndex_))) + 1 )*( maximumHighlightIndex_ )/2 ); }
