// $Id: TaskToolTip.cpp,v 1.43 2012/12/20 10:07:16 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "TaskToolTip.h"
#include "CustomPixmap.h"
#include "Debug.h"
#include "IconSize.h"
#include "Effects.h"
#include "Options.h"
#include "QtUtil.h"
#include "Singleton.h"
#include "SvgEngine.h"
#include "XmlOptions.h"

#include <QtGui/QLayout>
#include <QtGui/QPainter>
#include <QtGui/QStyleOptionFrame>
#include <QtGui/QToolTip>

#include "X11Util.h"
#include "CompositeEngine.h"

static unsigned int debugLevel(1);

//_______________________________________________________________________
TaskToolTip::TaskToolTip( QWidget* parent, const TaskAttributes& attributes ):
    TRANSPARENCY::TransparentWidget( parent, Qt::ToolTip | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint ),
    delay_( 0 ),
    flags_( None ),
    pixmapSize_( IconSize( IconSize::Maximum ) ),
    margin_( 10 ),
    useStyle_( true ),
    useSvg_( false )
{

    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();

    // attributes
    setAttributes( attributes );

}

//__________________________________________________
void TaskToolTip::setAttributes( const TaskAttributes& attributes )
{

    // try load pixmap from cache
    Cache::const_iterator iter( cache_.find( attributes.iconName() ) );
    if( iter != cache_.end() ) _setPixmap( iter.value() );
    else {

        // if not found, create a new pixmap,
        CustomPixmap pixmap( attributes.iconName() );
        if( pixmap.isNull() ) pixmap = CustomPixmap().empty( pixmapSize_ );
        else pixmap = pixmap.scaled( pixmapSize_, Qt::KeepAspectRatio, Qt::SmoothTransformation );

        // insert to pixmap
        cache_.insert( attributes.iconName(), pixmap );

        // assign to local
        _setPixmap( pixmap );

    }

    // title and comment
    _setTitle( attributes.name() );
    _setComments( attributes.comments() );

    // update tooltip size
    _adjustSize();

}

//____________________________________
void TaskToolTip::show( void )
{
    Debug::Throw( debugLevel, "TaskToolTip::Show.\n" );
    stopTimers();
    if( !flag( Visible ) ) showTimer_.start( delay_, this );
    else {
        setFlag( Visible, true );
        QWidget::show();
    }

}

//____________________________________
void TaskToolTip::hide( bool now )
{

    Debug::Throw( debugLevel, "TaskToolTip::Hide.\n" );
    stopTimers();
    QWidget::hide();
    if( now ) setFlag( Visible, false );
    else hideTimer_.start( delay_, this );

}

//____________________________________
void TaskToolTip::stopTimers( void )
{

    Debug::Throw( debugLevel, "TaskToolTip::StopTimers.\n" );
    showTimer_.stop();
    hideTimer_.stop();

}

//____________________________________
void TaskToolTip::_updateConfiguration( void )
{
    Debug::Throw( "TaskToolTip::_updateConfiguration.\n" );

    // transparency
    bool transparent(  XmlOptions::get().get<bool>( "TRANSPARENT_TOOLTIP" ) );
    if( transparent )
    {

        _setTransparent( true );
        setWindowOpacity( XmlOptions::get().get<double>( "TRANSPARENCY_OPACITY" )/255 );

    } else {

        _setTransparent( false );
        _setTintColor( QWidget().palette().color( QPalette::Window ) );
        setWindowOpacity( 1 );

    }

    // pixmap size
    int size = (int)( XmlOptions::get().get<int>( "TOOLTIP_HEIGHT" ) );
    pixmapSize_ = QSize( size, size );

    // svg
    _setUseSvg( XmlOptions::get().get<bool>( "USE_SVG" ) );
    _setUseStyle( XmlOptions::get().get<bool>( "USE_STYLED_TOOLTIPS" ) );

    setProperty( "_KDE_NET_WM_SKIP_SHADOW", !_useStyle() );

    // update foreground color
    if( _useStyle() )
    {

        // change palete
        setPalette( QToolTip::palette() );
        setBackgroundRole( QPalette::ToolTipBase );
        setForegroundRole( QPalette::ToolTipText );

    } else {

        setPalette( QWidget::palette() );
        setBackgroundRole( QPalette::Window );
        setForegroundRole( QPalette::WindowText );

        QColor foregroundColor( this->foregroundColor() );
        QPalette palette( TaskToolTip::palette() );
        if( foregroundColor.isValid() )
        {
            palette.setColor( foregroundRole(), foregroundColor );
            setPalette( palette );
        }

    }

    // fonts
    largeFont_ = font();
    largeFont_.setPointSize( 20 );
    largeFont_.setBold( true );

    normalFont_ = font();

    // rounded flags
    setFlag( Rounded, XmlOptions::get().get<bool>("ROUND_TOOLTIP") && !_useStyle() );

    // margin
    margin_ = 10 + XmlOptions::get().get<int>( "MARGIN" );

    // works on X11 only
    X11Util::get().changeProperty( *this, X11Util::_NET_WM_STATE_SKIP_TASKBAR, true );
    X11Util::get().changeProperty( *this, X11Util::_NET_WM_STATE_SKIP_PAGER, true );

    return;
}

//__________________________________________________
void TaskToolTip::_paint( QPaintDevice& device, const QRect& rect )
{
    QPainter painter(&device);
    painter.setClipRect( rect );
    painter.setPen( palette().color( foregroundRole() ) );

    if( _useStyle() )
    {

        QStyleOptionFrame opt;
        opt.init(this);
        style()->drawPrimitive(QStyle::PE_PanelTipLabel, &opt, &painter, this );

    } else {

        if( TRANSPARENCY::CompositeEngine::get().isEnabled() )
        {
            painter.setRenderHints(QPainter::SmoothPixmapTransform);
            painter.setCompositionMode(QPainter::CompositionMode_Source );
        }

        if( _useSvg() && SVG::SvgEngine::get().isValid() )
        { painter.drawPixmap( QPoint(0,0), SVG::SvgEngine::get().get( size() ) ); }

    }

    painter.setCompositionMode(QPainter::CompositionMode_SourceOver );

    // pixmap
    if( !_pixmap().isNull() ) painter.drawPixmap( margin_, margin_, _pixmap() );

    QPixmap pixmap( size() );
    pixmap.fill( Qt::transparent );
    {
        QPainter painter( &pixmap );
        painter.setClipRect( rect );
        painter.setPen( palette().color( foregroundRole() ) );

        // font metrics
        QFontMetrics large_metrics( largeFont_ );
        QFontMetrics normal_metrics( normalFont_ );

        // title
        painter.setFont( largeFont_ );
        painter.drawText( 2*margin_ + _pixmap().width(), margin_ + large_metrics.lineSpacing(), _title() );

        // comments
        painter.setFont( normalFont_ );
        painter.drawText( 2*margin_ + _pixmap().width(), 2*margin_ + large_metrics.lineSpacing() + normal_metrics.lineSpacing(), _comments() );
        painter.end();
    }

    if( shadowOffset() > 0 && shadowColor().isValid() && !_useStyle() )
    {
        QImage image( pixmap.toImage() );
        TRANSPARENCY::Effects::shadowBlur(image, shadowOffset(), shadowColor() );
        painter.drawImage( TaskToolTip::rect().topLeft(), image);
    }

    painter.drawPixmap( TaskToolTip::rect().topLeft(), pixmap );

}

//__________________________________________________
void TaskToolTip::timerEvent( QTimerEvent* event )
{

    if( event->timerId() == showTimer_.timerId() )
    {

        setFlag( Visible, true );
        QWidget::show();
        showTimer_.stop();

    } else if( event->timerId() == hideTimer_.timerId() ) {

        setFlag( Visible, false );
        hideTimer_.stop();
        if( isVisible() ) QWidget::hide();

    } else return QWidget::timerEvent( event );

}

//__________________________________________________
void TaskToolTip::resizeEvent( QResizeEvent* )
{
    if( flag( Rounded ) ) setMask( RoundedRegion( rect(), RoundedRegion::All ) );
    if( _blurEnabled() && !_useStyle() )
    {

        static const int margin( 10 );
        _updateBlurRegion( rect().translated( mapTo( window(), QPoint(0,0) ) ).adjusted( margin, margin, -margin, -margin ) );

    }
}

//__________________________________________________
void TaskToolTip::_adjustSize( void )
{

    QFontMetrics large_metrics( largeFont_ );
    QFontMetrics normal_metrics( normalFont_ );

    int width = margin_ + _pixmap().width() + margin_ +
        qMax( large_metrics.width(_title()), normal_metrics.width( _comments() ) ) +
        margin_;

    int height = margin_ + qMax(
        _pixmap().height(),
        large_metrics.lineSpacing() + margin_ + normal_metrics.lineSpacing() ) +
        margin_;

    size_ = QSize( width, height );

}
